//Copyright Plasmatronics pty ltd 2023-2024 +61394869902
import Menu from './menu.js';
import Properties from './properties.js';
export default class Controller {
  constructor(model, view) {
    this.model = model;
    this.view = view;
    this.menu = new Menu();
    this.dingoStatus = null;
    this.dingoHistory = null;
    this.historyIndex = 0;
    this.uploadStatus = null;
    this.uploadProgress = null;
    this.historyUpdateRate = 60000;
    this.statusUpdateRate = 5000;
    this.request = 'dingoGetStatus';
    this.requestTimer = null;
    this.menuNode = "ROOT";
    this.menuEvent(this.menuNode);

    // event handlers
    let classInstance = this;
    this.view.on('menuEvent', function(ev) { classInstance.menuEvent(ev) });
    this.model.on('dingoGetStatusDone', (status) => {
      classInstance.dingoStatus = status;
      classInstance.updateView(null);
      classInstance.startRequestTimer(this.statusUpdateRate);
    });
    this.model.on('dingoSetConfigDone', (response) => {
      classInstance.setConfigDone(response);
      this.request = 'dingoGetStatus';
      classInstance.startRequestTimer(this.statusUpdateRate);
    });
    this.model.on('dingoGetHistoryDone', (history) => {
      classInstance.dingoHistory = history;
      classInstance.updateView(null);
      classInstance.startRequestTimer(this.historyUpdateRate);
    });
    this.model.on('dingoUploadProgress', (progress) => {
      this.view.popup.showUploadProgress(progress);
    });
    this.model.on('dingoUploadDone', (status) => {
      this.view.popup.showUploadDone(status);
    });
  }

  getprprty(register) {
    let prprty = this.dingoStatus.find(el => el.reg == register);	  
    if (prprty == undefined) {
      return {reg : "xxxxxx", label : "DNET error", units : "", format : "str", value: register}
    };
    return prprty;
  }
  
  updateInfo(menu) {
    let infoContent = [];
    if (menu.activeItem.par == 'HIST') {
      if (this.dingoHistory != null) {
        let history = this.dingoHistory[this.historyIndex];
        infoContent.push("Day " + (this.historyIndex + 1).toString());
        history.forEach((property) => {
          infoContent.push(property.label + "<br>" + property.value + " " +
                           property.units);
        });
      } else {
        infoContent = [ "Please Wait" ];
      }
    } else if (this.dingoStatus != null) {
      let register = menu.activeItem.reg;
      let property = this.dingoStatus.find(el => el.reg == register);
      if (property == undefined) {
        infoContent = [ "DNET Error" ];
		infoContent.push("~" + menu.activeItem.btn + "."+ register + "~");
      } else {
        infoContent = [ property.label, property.value + " " + property.units];
        let cfgtmr = this.dingoStatus.find(el => el.reg == 'cfgtmr');
        if ((property.cfg) && (cfgtmr.value == 0)) {
          infoContent.push("Press Configuration Button To Update");
		  
      } else if (property.reg == 'dntver') {
            infoContent = [property.value];
            infoContent.push(this.getprprty('rsemod').value);
            infoContent.push(this.getprprty('rsteip').value);
            infoContent.push(this.getprprty('rswmod').value);
            infoContent.push(this.getprprty('rstwip').value);
            infoContent.push(this.getprprty('dntbtm').value);
            infoContent.push("For factory reset, hold down and release config button, then reset power." );

      } else if (property.reg == 'dstats') {
            infoContent = ["<nobr>" + this.getprprty('wifsid').value + "</nobr>"] 
            let prptyB = this.getprprty('stemod');
            let prptyC = this.getprprty('ethipa');
            let prptyD = this.getprprty('duplex');
            infoContent.push("Ethernet:<br>~" + prptyB.value +"~<br>IP Address: " + prptyC.value+"<br>" + prptyD.value+"<br>--");
			
            prptyB = this.getprprty('stwmod');
            prptyC = this.getprprty('wifipa');
            infoContent.push("WIFI:<br>~" + prptyB.value+"~<br>IP Address: " + prptyC.value);
    }
      }
    } else {
      infoContent = [ "Network Error" ];
    }
    return infoContent;
  }

  startUpload(fileList) {
    this.uploadFile(fileList[0]);
  }

  uploadFile(file) {
    this.stopRequestTimer();
    this.dingoStatus = null;
    this.dingoHistory = null;
    this.model.emit('dingoUpload', file);
  }

  updateView(ev) {
	let menu = this.menu.update(this.menuNode);
    if (ev != null) {
		this.menuNode = ev;
		menu = this.menu.update(this.menuNode);
		this.view.renderMenu(menu);
    
		if (this.isConfigEnabled()) {
		  let itm=this.menu.findItem(ev)
		  if (itm != null) {
			let pop=itm.dialog;
			if (pop != null) {
				if (pop == "file") {	//if (ev == "UPDT") 
					this.view.popup.showUpload((fileList) => {
					this.startUpload(fileList);
					});
					return;
				}

				let reg=itm.reg;
				let lst=itm.lst;
				var classInstance = this;
				classInstance.register = reg;

				if (reg!=null) {
					let stat = this.dingoStatus.find(el => el.reg == reg);
					let label = Properties.find(reg).label;
					let fn = null;

				//((ev == "MBEN")) ((ev == "WIFM")) ((ev == "ETHM"))  (ev == "DPLX")) 
					if (pop == "rbtn"){
						//fn = this.view.popup.showRadio;
						this.view.popup.showRadio(label, stat.value, lst, (value) => {
							let config = {};
							config[classInstance.register] = value;
							classInstance.model.emit('dingoSetConfig', config);
						});
						return;
						//this.view.popup.showRadio(
						//	label, stat.value, lst, (reg,value) => {
						//	classInstance.model.emit('dingoSetConfig', {reg : value});
						//});
						//return;						
					}; 
				//((ev == "EIPA") || (ev == "EIPG") || (ev == "EIPM") ||
				//  (ev == "WIPA") || (ev == "WIPG") || (ev == "WIPM") ||
				//  (ev == "EIPS")) {
					if (pop == "ip4"){
						//fn = this.view.popup.showInputIP4;
						this.view.popup.showInputIP4(label, stat.value, lst, (value) => {
							let config = {};
							config[classInstance.register] = value;
							classInstance.model.emit('dingoSetConfig', config);
						});
						return;
						//this.view.popup.showInputIP4(label, stat.value, lst, (reg,value) => {
						//classInstance.model.emit('dingoSetConfig', {reg : value}); 
						//});
						//return;						
					};
					if (pop == "text"){ //((ev == "WIID") || (ev == "WIPW") || (ev == "EHST")) {
						//fn = this.view.popup.showInputText;
						this.view.popup.showInputText(label, stat.value,lst, (value) => {
							let config = {};
							config[classInstance.register] = value;
							classInstance.model.emit('dingoSetConfig', config);
						});
						return;
						
						//this.view.popup.showInputText(label, stat.value, lst, (reg,value) => {
						//classInstance.model.emit('dingoSetConfig', {reg : value}); 
						//});
						//return;						
					};
//					fn(label, stat.value, lst, (value) => {
//						classInstance.model.emit('dingoSetConfig', {reg : value}); 
//					});
//					return;
				}
			}
		  }
		}
	}
    let info = this.updateInfo(menu);
    this.view.renderInfo(info);
  }

  isConfigEnabled() {
    if (this.dingoStatus != null) {
      let cfgtmr = this.dingoStatus.find(el => el.reg == 'cfgtmr');
      if (cfgtmr.value > 10 * 10) { // config timer is in 100 ms intervals
        return true;
      }
    }
    return false;
  }

  stopRequestTimer() {
    if (this.requestTimer != null)
      clearTimeout(this.requestTimer);
    this.requestTimer = null;
  }

  startRequestTimer(timeout) {
    this.stopRequestTimer();
    if (this.isConfigEnabled() == false)
      this.view.popup.hidePrompt();
    let classInstance = this;
    this.requestTimer = setTimeout(function() {
      classInstance.model.emit(classInstance.request);
      classInstance.stopRequestTimer();
    }, timeout);
  }

  setConfigDone(response) {
    console.log("setConfigDone");
    console.log(response);
    if(response != null){
      this.view.popup.hidePrompt();
    }
  }

  menuEvent(ev) {
    if ((ev == 'HIST') || (ev == 'PREV') || (ev == 'NEXT')) {
      if (ev == 'PREV')
        this.historyIndex--;
      if (ev == 'NEXT')
        this.historyIndex++;
      if (this.historyIndex < 0)
        this.historyIndex = 99;
      if (this.historyIndex > 99)
        this.historyIndex = 0;
      this.updateView(ev);
      if (this.dingoHistory == null) {
        this.request = 'dingoGetHistory';
        this.startRequestTimer(0); // request history immediately if not loaded
      }
    } else {
      this.updateView(ev);
      this.request = 'dingoGetStatus';
      this.startRequestTimer(0); // request status immediately if button pressed
      this.dingoHistory = null; // ensure history data is re-loaded on select
    }
  }
}
