//Copyright Plasmatronics pty ltd 2023-2024 +61394869902
export default class Format {

  static enumerate(value, enumerationArray) {
    var number = Number(value);
    if ((number >= 0) && (number < enumerationArray.length)) {
      return enumerationArray[number];
    } else {
      return "Illegal Value";
    }
  }

  // populate the property with formated value
  static format(property, value) {
    // let property = {};
    // property.format = format;
    if(value == null){
      property.value = "";
      property.units = "No Data Available";
    } else if(property.format == "str"){
      property.value = value;
    } else if(property.format == "hide"){
      property.value = "";
    } else if (property.format == "milli") {
      property.value = (Number(value) * 0.001).toFixed(1);
    } else if (property.format == "deci") {
      property.value = (Number(value) * 0.1).toFixed(1);
    } else if (property.format == "time") {
      if (value == 255) {
        property.value = ""
        property.units = "Not Reached Float"
      } else {
        let number = Number(value);
        let hour = Math.floor(number / 10.0);
        let minute = Math.floor((number - hour * 10) * 6);
        property.value = hour.toString().padStart(2, '0') + ":" +
                     minute.toString().padStart(2, '0');
      }
    } else if (property.format == "prog") {
      property.units = this.enumerate(value, [
        "Flooded battery, No night light, Low battery disconnect",
        "Sealed battery, No night light, Low battery disconnect",
        "Flooded battery, Night light, Low battery disconnect",
        "Sealed battery, Night light, Low battery disconnect",
        "User configured settings"
      ]);
      property.value = value.toFixed(0);
    } else if (property.format == "volt") {
      property.value = this.enumerate(value, [ "12", "24", "32", "36", "48" ]);
    } else if (property.format == "unit") {
      property.value = value.toFixed(0);
    } else if (property.format == "ldst") {
      property.units = this.enumerate(value, [ "Connected", "Disconnected" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "gnst") {
      property.units = this.enumerate(value, [ "Off", "Running" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "evst") {
      property.units = this.enumerate(value, [ "Off", "Active" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "cpst") {
      property.units = this.enumerate(value, [ "OK", "Tripped" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "lpst") {
      property.units = this.enumerate(value, [ "OK", "Tripped" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "alst") {
      property.units = this.enumerate(value, [ "Off", "Active" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "regm") {
      property.units = this.enumerate(
          value, [ "Boost", "Equalisation", "Absorbtion", "Float" ]);
      property.value = value.toFixed(0);
    } else if (property.format == "temp") {
      if (value == -128) {
        property.value = "";
        property.units = "Sensor Not Present";
      } else {
        property.value = value.toFixed(0);
        property.units = "Celcius";
      }
    } else {
      property.value = "Illegal Format";
    }
    return property;
  }
}
