//Copyright Plasmatronics pty ltd 2023-2024 +61394869902
import Dom from './dom.js';

export default class Popup {
  constructor() {
    this.dom = new Dom();
    this.active = false;
    this.allowCancel = true;
    this.FileList = null;
    this.popupdiv = document.getElementById("popupdiv");
    this.popupcont = document.getElementById("popupcont");
    this.hidePrompt = this.hidePrompt.bind(this);
    this.showPrompt = this.showPrompt.bind(this);
    this.validateIP4 = this.validateIP4.bind(this);
    this.showInputIP4 = this.showInputIP4.bind(this);
    this.showInputText = this.showInputText.bind(this);
    this.showUpload = this.showUpload.bind(this);
  }

  hidePrompt() {
    if (this.allowCancel){
      this.active = false;
      this.popupdiv.style.display = "none";
      let exisitingTable = document.getElementById('popuptable');
      if (exisitingTable != null) {
        exisitingTable.remove();
      }
    }
  }

  showPrompt(domObjects) {
    let classInstance = this;
    this.active = true;
    this.popupdiv.style.display = "block";
    let table = document.createElement('div');
    table.setAttribute("id", "popuptable");
    if (this.allowCancel)
      domObjects.push(classInstance.dom.button("CLOSE", "cancel",
                                               classInstance.hidePrompt));
    domObjects.forEach(function(domObject) {
      table.append(classInstance.dom.rowcell(domObject, "trow", "tcell"));
    });
    let exisitingTable = document.getElementById('popuptable');
    if (exisitingTable == null) {
      this.popupcont.append(table);
    } else {
      exisitingTable.replaceWith(table);
    }
  }

  inputValid() {
    let input = document.getElementById('inputBox');
    if (input != null) {
      input.classList.remove('inputinvalid');
      input.classList.add('inputvalid');
    }
    let button = document.getElementById('saveButton');
    if (button != null) {
      button.disabled = false;
    }
  }

  inputInvalid() {
    let input = document.getElementById('inputBox');
    if (input != null) {
      input.classList.remove('inputvalid');
      input.classList.add('inputinvalid');
    }
    let button = document.getElementById('saveButton');
    if (button != null) {
      button.disabled = true;
    }
  }

  isValidIP4(ip) {
    var ipformat =
        /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
    if (ip.match(ipformat)) {
      return true;
    }
    return false;
  }

  validateIP4() {
    let input = document.getElementById('inputBox');
    if (input != null) {
      if (this.isValidIP4(input.value)) {
        this.inputValid();
      } else {
        this.inputInvalid();
      }
    }
  }

  showUploadProgress(message) {
    this.allowCancel = false;
    this.showPrompt([
      this.dom.text("Update Firmware", "p"), this.dom.text("Progress", "p"),
      this.dom.text(message, "p")
    ]);
  }

  showUploadDone(message){
    this.showPrompt([
      this.dom.text("Update Firmware", "p"), this.dom.text("Progress", "p"),
      this.dom.text(message, "p"), this.dom.button("DONE", "done",
                                               this.hidePrompt)
    ]);
    this.allowCancel = true;
  }

  showUpload(callback) {
    let inputBox = this.dom.input("inputBox");
    inputBox.setAttribute("type", "file");
    inputBox.setAttribute("id", "filename");
    if (this.fileList == null) {
      inputBox.addEventListener('change', (ev) => {
        if (typeof ev.target.files.length != undefined) {
          if (ev.target.files.length == 1) {
            this.fileList = ev.target.files;
            let uploadButton = this.dom.button("UPLOAD", "uploadButton",
                                               () => callback(this.fileList));
            this.showPrompt([
              this.dom.text("Update Firmware", "p"), inputBox, uploadButton
            ]);
          }
        } else
          this.fileList = null;
      });
      this.showPrompt([ this.dom.text("Update Firmware", "p"), inputBox ]);
    } else {
      console.log(this.fileList);
      console.log(inputBox);
      inputBox.files = this.fileList;
      let uploadButton = this.dom.button("UPLOAD", "uploadButton",
                                         () => callback(this.fileList));
      this.showPrompt(
          [ this.dom.text("Update Firmware", "p"), inputBox, uploadButton ]);
    }
  }

  showInput(label, value, saveEvent, validateEvent) {
    let inputBox = this.dom.input("inputBox", validateEvent);
    inputBox.setAttribute("maxlength", 16);
    inputBox.value = value;
    let saveButton =
        this.dom.button("SAVE", "saveButton", () => saveEvent(inputBox.value));
    this.showPrompt([ this.dom.text(label, "p"), inputBox, saveButton ]);
  }

  showInputIP4(label, value, x, saveEvent) {
    this.showInput(label, value, saveEvent, this.validateIP4);
    this.validateIP4();
  }

  showInputText(label, value, x, saveEvent) {
    this.showInput(label, value, saveEvent, null);
  }

  showRadio(label, value, valueList, saveEvent) {
    let classInstance = this;
    let domObjects = [ classInstance.dom.text(label, "p") ];
    valueList.forEach((obj) => {
      let radioButton = classInstance.dom.radio(obj, (obj == value));
      domObjects.push(radioButton);
    });
    let saveButton = this.dom.button("SAVE", "saveButton", () => {
      let radios = Array.from(document.getElementsByName('radio'));
      let checked = radios.find(radio => radio.checked == true)
      saveEvent(checked.id);
    });
    domObjects.push(saveButton);
    this.showPrompt(domObjects);
  }

  showInputSaving() {}
}
