//Copyright Plasmatronics pty ltd 2023-2024 +61394869902
import Dom from './dom.js';
import EventEmitter from './eventemitter.js';
import Popup from './popup.js';

export default class View extends EventEmitter {

  constructor() {
    super();
    this.fileList = null;
    this.popup = new Popup();
    this.dom = new Dom();
    document.addEventListener('keydown', (ev) => {
      if (ev.key == 'Escape') {
        this.popup.hidePrompt();
      }
    });
    document.addEventListener('pointerdown', (ev) => {
      if (ev.target.id == 'popupdiv') {
        this.popup.hidePrompt();
      }
    });
  }

  renderInfo(infoArray) {
    let classInstance = this;
    let content = document.createElement('div');
    content.setAttribute("id", "infodiv");
    let domObjects = [];
    infoArray.forEach(
        (info) => { domObjects.push(classInstance.dom.text(info, "p")); });
    let table = classInstance.dom.table("infotable", domObjects)
    content.append(table);
    document.getElementById('infodiv').replaceWith(content);
  }

  renderMenu(menuArray) {
    let classInstance = this;
    let menu = document.createElement('div');
    menu.setAttribute("id", "menudiv");
    let domObjects = [];
    menuArray.menuItems.forEach((menuItem) => {
      let button =
          classInstance.dom.button(menuItem.label, menuItem.btn, () => {
            classInstance.emit('menuEvent', menuItem.btn);
          });
      if (menuArray.activeItem == menuItem)
        button.classList.add('menuactive');
      domObjects.push(button);
    });
    let table = classInstance.dom.table("menutable", domObjects)
    menu.append(table);
    document.getElementById('menudiv').replaceWith(menu);
  }

}
